<?php
/**
 * A class for directory reading and getting its contents
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2012 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class FrameworkDirectoryPointer extends BaseFrameworkSystem {
	/**
	 * The current path we are working in
	 */
	private $pathName = '';

	/**
	 * The directory pointer
	 */
	private $dirPointer = NULL;

	/**
	 * Protected constructor
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Destructor for cleaning purposes, etc
	 */
	public function __destruct() {
		// Is there a resource pointer? Then we have to close the directory here!
		if (is_resource($this->getPointer())) {
			// Try to close a directory
			$this->closeDirectory();
		}

		// Call the parent destructor
		parent::__destruct();
	}

	/**
	 * Create a directory pointer based on the given path. The path will also
	 * be verified here.
	 *
	 * @param	$pathName		The path name we shall pass to opendir()
	 * @param	$inConstructor	If we are in de/con-structor or from somewhere else
	 * @return	$pointerInstance	A prepared instance of FrameworkDirectoryPointer
	 * @throws	PathIsEmptyException	If the provided path name is empty
	 * @throws	InvalidPathStringException	If the provided path name is not a string
	 * @throws	PathIsNoDirectoryException	If the provided path name is not valid
	 * @throws	PathReadProtectedException	If the provided path name is read-protected
	 * @throws	DirPointerNotOpenedException	If opendir() returns not a directory resource
	 */
	public static final function createFrameworkDirectoryPointer ($pathName, $inConstructor = false) {
		// Some pre-sanity checks...
		if (is_null($pathName)) {
			// No pathname given
			if ($inConstructor) {
				return NULL;
			} else {
				throw new PathIsEmptyException(null, self::EXCEPTION_UNEXPECTED_EMPTY_STRING);
			}
		} elseif (!is_string($pathName)) {
			// Is not a string
			if ($inConstructor) {
				return NULL;
			} else {
				throw new InvalidPathStringException(null, self::EXCEPTION_INVALID_STRING);
			}
		} elseif (!is_dir($pathName)) {
			// Not a directory
			if ($inConstructor) {
				return NULL;
			} else {
				throw new PathIsNoDirectoryException($pathName, self::EXCEPTION_INVALID_PATH_NAME);
			}
		} elseif (!is_readable($pathName)) {
			// Not readable
			if ($inConstructor) {
				return NULL;
			} else {
				throw new PathReadProtectedException($pathName, self::EXCEPTION_READ_PROTECED_PATH);
			}
		}

		// Try to open a handler
		$dirPointer = @opendir($pathName);
		if (!is_resource($dirPointer)) {
			// Something bad happend
			if ($inConstructor) {
				return NULL;
			} else {
				throw new DirPointerNotOpenedException($pathName, self::EXCEPTION_DIR_POINTER_INVALID);
			}
		}

		// Create new instance
		$pointerInstance = new FrameworkDirectoryPointer();

		// Set directory pointer and path name
		$pointerInstance->setPointer($dirPointer);
		$pointerInstance->setPathName($pathName);

		// Return the instance
		return $pointerInstance;
	}

	/**
	 * Read raw lines of data from a directory pointer and return the data
	 *
	 * @return	string	Directory and/or file names read from the current directory pointer
	 */
	public function readRawDirectory () {
		// Read data from the directory pointer and return it
		return readdir($this->getPointer());
	}

	/**
	 * Read lines from the current directory pointer except some parts
	 *
	 * @param	$except		Some parts of a directory we want to ignore. Valid: directory and file names, other values will be silently ignored
	 * @return	string		Directory and/or file names read from the current directory pointer
	 */
	public function readDirectoryExcept (array $except = array('.', '..')) {
		if (count($except) == 0) {
			// No exception given, so read all data
			self::createDebugInstance(__CLASS__)->debugOutput('DIRECTORY: No exceptions given, please use readRawDirectory() instead!');
			return $this->readRawDirectory();
		} // END - if

		// Read a raw line...
		$rawLine = $this->readRawDirectory();

		// Shall we exclude directories?
		if ((!is_null($rawLine)) && ($rawLine !== false) && (!in_array($rawLine, $except))) {
			// Return read data
			//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('DIRECTORY: rawLine[' . gettype($rawLine) . ']=' . $rawLine);
			return $rawLine;
		} elseif ((!is_null($rawLine)) && ($rawLine !== false)) {
			// Exclude this part
			//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('DIRECTORY: rawline[' . gettype($rawLine) . ']=' . $rawLine . ' - Recursive call!');
			return $this->readDirectoryExcept($except);
		}

		// End pointer reached
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('DIRECTORY: Returning NULL!');
		return NULL;
	}

	/**
	 * Close a directory source and set it's instance to null and the path name
	 * to empty
	 *
	 * @return	void
	 */
	public function closeDirectory () {
		// Close the directory pointer and reset the instance variable
		@closedir($this->getPointer());
		$this->setPointer(NULL);
		$this->setPathName('');
	}

	/**
	 * Setter for the directory pointer
	 *
	 * @param	$dirPointer		The directory resource
	 * @return	void
	 */
	public final function setPointer ($dirPointer) {
		// Sanity-check if pointer is a valid directory resource
		if (is_resource($dirPointer) || is_null($dirPointer)) {
			// Is a valid resource
			$this->dirPointer = $dirPointer;
		} else {
			// Throw exception
			throw new InvalidResourceException($this, self::EXCEPTION_INVALID_RESOURCE);
		}
	}

	/**
	 * Getter for the directory pointer
	 *
	 * @return	$dirPointer		The directory pointer which shall be a valid directory resource
	 */
	public final function getPointer () {
		return $this->dirPointer;
	}

	/**
	 * Setter for path name
	 *
	 * @param	$pathName	The new path name
	 * @return	void
	 */
	public final function setPathName ($pathName) {
		$pathName = (string) $pathName;
		$this->pathName = $pathName;
	}

	/**
	 * Getter for path name
	 *
	 * @return	$pathName	The current path name
	 */
	public final function getPathName () {
		return $this->pathName;
	}
}

// [EOF]
?>
